"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from typing import Any, List, Type, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="PrecisionRecallCurve")


@_attrs_define
class PrecisionRecallCurve:
    """
    Attributes:
        thresholds (List[float]):
        precisions (List[float]):
        recalls (List[float]):
        auc (float):
    """

    thresholds: List[float]
    precisions: List[float]
    recalls: List[float]
    auc: float
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        thresholds = self.thresholds

        precisions = self.precisions

        recalls = self.recalls

        auc = self.auc

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "thresholds": thresholds,
                "precisions": precisions,
                "recalls": recalls,
                "auc": auc,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        thresholds = cast(List[float], d.pop("thresholds"))

        precisions = cast(List[float], d.pop("precisions"))

        recalls = cast(List[float], d.pop("recalls"))

        auc = d.pop("auc")

        precision_recall_curve = cls(
            thresholds=thresholds,
            precisions=precisions,
            recalls=recalls,
            auc=auc,
        )

        precision_recall_curve.additional_properties = d
        return precision_recall_curve

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
