"""
This file is generated by the openapi-python-client tool via the generate_api_client.py script

It is a customized template from the openapi-python-client tool's default template:
https://github.com/openapi-generators/openapi-python-client/blob/861ef5622f10fc96d240dc9becb0edf94e61446c/openapi_python_client/templates/model.py.jinja

The main change is:
- Fix typing issues
"""

# flake8: noqa: C901

from typing import TYPE_CHECKING, Any, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.memoryset_neighbor_metrics_lookup_score_metrics import MemorysetNeighborMetricsLookupScoreMetrics


T = TypeVar("T", bound="MemorysetNeighborMetrics")


@_attrs_define
class MemorysetNeighborMetrics:
    """
    Attributes:
        lookup_score_metrics (MemorysetNeighborMetricsLookupScoreMetrics):
    """

    lookup_score_metrics: "MemorysetNeighborMetricsLookupScoreMetrics"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        lookup_score_metrics = self.lookup_score_metrics.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "lookup_score_metrics": lookup_score_metrics,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: dict[str, Any]) -> T:
        from ..models.memoryset_neighbor_metrics_lookup_score_metrics import MemorysetNeighborMetricsLookupScoreMetrics

        d = src_dict.copy()
        lookup_score_metrics = MemorysetNeighborMetricsLookupScoreMetrics.from_dict(d.pop("lookup_score_metrics"))

        memoryset_neighbor_metrics = cls(
            lookup_score_metrics=lookup_score_metrics,
        )

        memoryset_neighbor_metrics.additional_properties = d
        return memoryset_neighbor_metrics

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
