.white {
  background-color: white;
}
.centered input {
  margin: auto;
}
.centered p {
  text-align: center;
}
.button {
  display: inline-block;
  max-width: 250px;
  background-color: #2b9a66;
  color: white;
  position: fixed;
  z-index: 1000;
  right: 36px;
  border-radius: 8px;
  top: 12px;
}
.margin-top {
  margin-top: 60px;
}
.header {
  position: fixed;
  z-index: 1000;
  height: 64px;
  left: 0;
  top: 0;
  border-radius: 0;
}

input[type='checkbox']:checked,
input[type='checkbox']:checked:hover,
input[type='checkbox']:checked:focus {
  background-color: #2b9a66;
  border-color: #2b9a66;
}
input[type='checkbox']:focus {
  border-color: #2b9a66;
}
.html-container:has(.no-padding) {
  padding: 0;
}

.progress-bar {
  background-color: #2b9a66;
}
.header .full {
  position: fixed !important;
  z-index: 1100;
  background-color: #e4e4e7;
  height: 68px;
}
